/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.jitterbit.util.collections.HashBag;
import org.jitterbit.util.collections.ICollectionFactory;

/**
 * Maintains a collection of <code>IExpandedNames</code> as a bi-directional
 * map. The names added are stored according to both their local names, and
 * their namespace URIs.
 * 
 * @author Torgil
 * 
 * @since 1.2
 * 
 */
public class ExpandedNameBag {

    private final HashBag<String, IExpandedName>    m_uriToExpandedName;
    private final HashBag<String, IExpandedName>    m_localNameToExpandedName;
    
    /**
     * Creates an empty ExpandedNameBag.
     * 
     */
    public ExpandedNameBag() {
        m_uriToExpandedName = new HashBag<String, IExpandedName>();
        m_uriToExpandedName.setCollectionFactory(ICollectionFactory.HASH_SET_FACTORY);
        m_localNameToExpandedName = new HashBag<String, IExpandedName>();
        m_localNameToExpandedName.setCollectionFactory(ICollectionFactory.HASH_SET_FACTORY);
    }

    /**
     * Adds a new <code>IExpandedName</code> to this bag.
     * 
     * @throws NullPointerException
     *             if <code>name</code> is <code>null</code>.
     */
    public void add(IExpandedName name) {
        m_uriToExpandedName.put(name.getNamespace(), name);
        m_localNameToExpandedName.put(name.getLocalName(), name);
    }

    /**
     * Adds the <code>IExpandedName</code> instances from the specified array.
     * 
     * @throws NullPointerException
     *             if <code>names</code> is null, or if <code>names</code>
     *             contains <code>null</code> elements.
     */
    public void addAll(IExpandedName[] names) {
        for (IExpandedName name : names) {
            add(name);
        }
    }

    /**
     * Adds a new <code>IExpandedName</code>, as created from the specified
     * string, to this bag
     * 
     * @param name
     *            a String representation of the name to add.
     * @param nameFactory
     *            an <code>XmlNameFactory</code> that will be asked to create
     *            an <code>IExpandedName</code> instance from the specified
     *            string.
     * @throws NullPointerException
     *             if <code>nameFactory</code> is null.
     * @throws IllegalArgumentException
     *             if the specified String could not be parsed into an
     *             <code>IExpandedName</code> by the specified
     *             <code>XmlNameFactory</code>.
     * 
     */
    public void add(String name, XmlNameFactory nameFactory) {
        IExpandedName expName = nameFactory.newExpandedName(name);
        add(expName);
    }
    
    /**
     * Adds new <code>IExpandedName</code> instances, as created from the
     * specified strings, to this bag
     * 
     * @param names
     *            an array holding the String representations of the names to
     *            add.
     * @param nameFactory
     *            an <code>XmlNameFactory</code> that will be asked to create
     *            <code>IExpandedName</code> instances from the specified
     *            strings.
     * @throws NullPointerException
     *             if <code>nameFactory</code> is null.
     * @throws IllegalArgumentException
     *             if any of the specified strings could not be parsed into an
     *             <code>IExpandedName</code> by the specified
     *             <code>XmlNameFactory</code>.
     * 
     */
    public void addAll(String[] names, XmlNameFactory nameFactory) {
        for (String name : names) {
            add(name, nameFactory);
        }
    }
    
    /**
     * Returns all unique namespace URIs of the names that are currently in this
     * bag.
     * 
     * @return an array of strings holding the namespace URIs of each distinct
     *         namespace URI that has been added to this bag. An empty array is
     *         returned if this bag is empty.
     */
    public String[] getNamespaces() {
        Set<String> keys = m_uriToExpandedName.keySet();
        return keys.toArray(new String[keys.size()]);
    }

    /**
     * Returns the number of unique namespace URIs that are currently in this
     * bag.
     * 
     */
    public int getNumberOfNamespaces() {
        return m_uriToExpandedName.size();
    }
    
    /**
     * Returns the total number of names that are currently stored in this bag.
     * 
     */
    public int getNumberOfNames() {
        int size = 0;
        for (String key : m_uriToExpandedName.keySet()) {
            List<? extends IExpandedName> values = m_uriToExpandedName.get(key);
            size += values.size();
        }
        return size;
    }
    
    /**
     * Returns the <code>IExpandedName</code> instances belonging to the
     * specified namespace URI that are currently in this bag.
     * 
     * @param uri
     *            the namespace URI the <code>IExpandedNames</code> of which
     *            to retrieve.
     * @return an array of all <code>IExpandedName</code> instances from this
     *         bag that belong to the specified namespace. An empty array is
     *         returned if this bag does not contain any names from the
     *         requested namespace.
     * 
     */
    public IExpandedName[] getNamesFromNamespace(String uri) {
        List<? extends IExpandedName> names = m_uriToExpandedName.get(uri);
        return names.toArray(new IExpandedName[names.size()]);
    }

    /**
     * Checks if this <code>ExpandedNameBag</code> contains the specified
     * <code>IExpandedName</code>.
     * 
     * @return <code>true</code> if and only if this bag contais the specified
     *         name.
     * @throws NullPointerException
     *             if <code>name</code> is null.
     */
    public boolean contains(IExpandedName name) {
        String uri = name.getNamespace();
        return m_uriToExpandedName.contains(uri, name);
    }
    
    /**
     * Returns all <code>IExpandedName</code> instances in this bag having the
     * specified local name.
     * 
     * @param localName
     *            a <code>String</code> holding the local name.
     * @return an array of all <code>IExpandedName</code> instances from this
     *         bag that has the specified local name. An empty array is returned
     *         if this bag does not contain any such names.
     */
    public IExpandedName[] getNamesFromLocalName(String localName) {
        List<IExpandedName> expanded = m_localNameToExpandedName.get(localName);
        return expanded.toArray(new IExpandedName[expanded.size()]);
    }

    /**
     * Checks if this bag contains any <code>IExpandedName</code> instances
     * with the specified local name.
     * 
     * @param localName
     *            a <code>String</code> holding the local name looked for.
     * @return <code>true</code> if and only of this bag contains at least one
     *         <code>IExpandedName</code> with the specified local name.
     */
    public boolean containsLocalName(String localName) {
        return !m_localNameToExpandedName.get(localName).isEmpty();
    }
    
    /**
     * Testing code.
     * 
     */
    public static void main(String[] args) {
        ExpandedNameBag bag = new ExpandedNameBag();
        String[] names = {
            "http://www.jitterbit.com/xml:Root",
            "http://www.jitterbit.com/xml:Child",
            "http://www.jitterbit.com/qa:Root",
            "http://www.jitterbit.com/qa:Root",
        };
        XmlNameFactory f = XmlNameFactory.nonConforming();
        for (String name : names) {
            IExpandedName exp = f.newExpandedName(name);
            bag.add(exp);
            System.out.println(bag.getNumberOfNamespaces());
            System.out.println(bag.getNumberOfNames());
        }
        System.out.println(Arrays.toString(bag.getNamespaces()));
        for (String uri : bag.getNamespaces()) {
            for (IExpandedName name : bag.getNamesFromNamespace(uri)) {
                System.out.println(name);
            }
        }
        IExpandedName test = f.newExpandedName("http://www.jitterbit.com/qa", "Root");
        System.out.println(bag.contains(test));
        System.out.println(bag.getNumberOfNames());
    }
}
