/*
 * Copyright (c) 1998-2006 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.io.File;
import java.io.Reader;

import org.w3c.dom.Element;

/**
 * Extracts the root element from an XML document.
 * 
 * @author Torgil
 * 
 * @since 1.2
 *
 */
public abstract class RootNodeExtractor {

    private static RootNodeExtractor    s_domImplementation;
    private static RootNodeExtractor    s_saxImplementation;
   
    /**
     * Extracts the root element from the XML document in the specified file.
     * 
     * @param xmlFile
     *            the XML file from which to extract the root node.
     * @throws KongaXmlException
     *             if the specified file could not be read, or if the file does
     *             not contain valid XML.
     */
    public abstract Element extractRoot(File xmlFile) throws KongaXmlException;

    /**
     * Extracts the root element from the XML document defined by the specified
     * <code>java.io.Reader</code>.
     * <p>
     * If the XML to be parsed is in the form of an in-memory string, this
     * method can be used as follows:
     * 
     * <pre>
     *    String xml = ...
     *    Reader reader = null;
     *    try {
     *        RootNodeExtractor extractor = ...
     *        reader = new StringReader(xml);
     *        extractor.extractRoot(reader);
     *    }
     *    finally {
     *        reader.close();
     *    }
     * </pre>
     * 
     * @param reader
     *            a <code>Reader</code> from which the XML to be parsed is
     *            read.
     * @throws KongaXmlException
     *             if the XML data could not be read from the specified
     *             <code>Reader</code>, or if the XML defined by the
     *             <code>Reader</code> is invalid.
     */
    public abstract Element extractRoot(Reader reader) throws KongaXmlException;

    /**
     * Returns a <code>RootNodeExtractor</code> that uses DOM to parse the XML
     * and extract root node information.
     * 
     * @return a <code>RootNodeExtractor</code> that uses a DOM parser to
     *         extract the root node.
     */
    public synchronized static RootNodeExtractor getDomExtractor() {
        if (s_domImplementation == null) {
            s_domImplementation = new DomRootNodeExtractor();
        }
        return s_domImplementation;
    }
    
    /**
     * Returns a <code>RootNodeExtractor</code> that uses SAX to parse the XML
     * and extract root node information.
     * <p>
     * <strong>Note:</strong> Due to the SAX nature of the parsing, the
     * <code>org.wc3.dom.Element</code> instance returned by the extractor
     * supports only a limited subset of the methods in the <code>Element</code>
     * interface. the following methods are supported:
     * <ul>
     * <li>getNodeType()</li>
     * <li>getTagName()</li>
     * <li>getNodeName()</li>
     * <li>getLocalName()</li>
     * <li>getNamespaceURI()</li>
     * <li>getPrefix()</li>
     * <li>hasAttributes()</li>
     * <li>hasAttribute(String)</li>
     * <li>getAttribute(String)</li>
     * </ul>
     * Calling any other method on the returned <code>Element</code> instance
     * is a no-op. The unsupported methods that have a return type return
     * <code>null</code>. None of the methods will result in an exception
     * being thrown.
     * 
     * @return a <code>RootNodeExtractor</code> that uses a DOM parser to
     *         extract the root node.
     */
    public synchronized static RootNodeExtractor getSaxExtractor() {
        if (s_saxImplementation == null) {
            s_saxImplementation = new SaxRootNodeExtractor();
        }
        return s_saxImplementation;
    }

    
    
    public static void main(String[] args) throws Exception {
        File file = new File("C:\\Temp\\XMLFile1.xml");
        extract(file, getDomExtractor(), "DOM");
        extract(file, getSaxExtractor(), "SAX");
    }
    
    private static void extract(File file, 
                                RootNodeExtractor extractor, 
                                String caption) throws Exception {
        System.out.println(caption);
        Element root = extractor.extractRoot(file);
        System.out.println(root.getNodeName());
        System.out.println(root.getTagName());
        XmlNameFactory nameFactory = XmlNameFactory.conforming();
        IExpandedName expName = nameFactory.newExpandedName(root);
        System.out.println(expName);
        IQualifiedName qName = nameFactory.newQualifiedName(root);
        System.out.println(qName);
    }
}
